@@ -1,13 +1,29 @@
+import numbers
 import os
 import re
-import types
-import pytest
-import numbers
 import tempfile
+import types
+
 import ipywidgets
-import numpy as np
-import discretisedfield as df
 import matplotlib.pyplot as plt
+import numpy as np
+import pytest
+
+import discretisedfield as df
+
+from .test_region import html_re as region_html_re
+
+html_re = (
+    r"<strong>Mesh</strong>\s*<ul>\s*"
+    rf"<li>{region_html_re}</li>\s*"
+    r"<li>n = .*</li>\s*"
+    r"(<li>bc = ([xyz]{1,3}|neumann|dirichlet)<li>)?\s*"
+    rf"(<li>subregions:\s*<ul>\s*(<li>{region_html_re}</li>\s*)+</ul></li>)?"
+    r"\s*<li>attributes:\s*<ul>\s*"
+    r"(\s*<li>(.*:.*|.*Mesh.*)</li>)+\s*"
+    r"</ul>\s*</li>\s*"
+    r"</ul>"
+)
 
 
 def check_mesh(mesh):
@@ -33,8 +49,11 @@ def check_mesh(mesh):
     assert len(mesh) > 0
 
     assert isinstance(repr(mesh), str)
-    pattern = r'^Mesh\(region=Region\(p1=\(.+\), p2=\(.+\)\), n=.+\)$'
-    assert re.search(pattern, repr(mesh))
+    pattern = r"^Mesh\(Region\(p1=\(.+\), p2=\(.+\)\), n=.+\)$"
+    assert re.match(pattern, str(mesh))
+
+    assert isinstance(mesh._repr_html_(), str)
+    assert re.match(html_re, mesh._repr_html_(), re.DOTALL)
 
     assert isinstance(mesh.indices, types.GeneratorType)
     assert isinstance(mesh.__iter__(), types.GeneratorType)
@@ -49,9 +68,9 @@ def check_mesh(mesh):
 
     assert isinstance(mesh.attributes, dict)
 
-    plane_mesh = mesh.plane('z', n=(2, 2))
+    plane_mesh = mesh.plane("z", n=(2, 2))
     assert isinstance(plane_mesh, df.Mesh)
-    assert plane_mesh.attributes['isplane']
+    assert plane_mesh.attributes["isplane"]
     assert 1 in plane_mesh.n
     assert len(plane_mesh) == 4
     assert all(isinstance(i, tuple) for i in plane_mesh)
@@ -67,53 +86,39 @@ def check_mesh(mesh):
 
 class TestMesh:
     def setup(self):
-        self.valid_args = [[(0, 0, 0), (5, 5, 5),
-                            [1, 1, 1], None],
-                           [(-1, 0, -3), (5, 7, 5),
-                            None, (1, 1, 1)],
-                           [(0, 0, 0), (5e-9, 5e-9, 5e-9),
-                            None, (1e-9, 1e-9, 1e-9)],
-                           [(0, 0, 0), (5e-9, 5e-9, 5e-9),
-                            (5, 5, 5), None],
-                           [(-1.5e-9, -5e-9, 0), (1.5e-9, -15e-9, -10e-9),
-                            None, (1.5e-9, 0.5e-9, 10e-9)],
-                           [(-1.5e-9, -5e-9, 0), (1.5e-9, -15e-9, -10e-9),
-                            (3, 10, 2), None],
-                           [(-1.5e-9, -5e-9, -5e-9), np.array((0, 0, 0)),
-                            None, (0.5e-9, 1e-9, 5e-9)],
-                           [(-1.5e-9, -5e-9, -5e-9), np.array((0, 0, 0)),
-                            (5, 5, 7), None],
-                           [[0, 5e-6, 0], (-1.5e-6, -5e-6, -5e-6),
-                            None, (0.5e-6, 2e-6, 2.5e-6)],
-                           [[0, 5e-6, 0], (-1.5e-6, -5e-6, -5e-6),
-                            (1, 10, 20), None],
-                           [(0, 125e-9, 0), (500e-9, 0, -3e-9),
-                            None, (25e-9, 25e-9, 3e-9)]]
+        self.valid_args = [
+            [(0, 0, 0), (5, 5, 5), [1, 1, 1], None],
+            [(-1, 0, -3), (5, 7, 5), None, (1, 1, 1)],
+            [(0, 0, 0), (5e-9, 5e-9, 5e-9), None, (1e-9, 1e-9, 1e-9)],
+            [(0, 0, 0), (5e-9, 5e-9, 5e-9), (5, 5, 5), None],
+            [
+                (-1.5e-9, -5e-9, 0),
+                (1.5e-9, -15e-9, -10e-9),
+                None,
+                (1.5e-9, 0.5e-9, 10e-9),
+            ],
+            [(-1.5e-9, -5e-9, 0), (1.5e-9, -15e-9, -10e-9), (3, 10, 2), None],
+            [(-1.5e-9, -5e-9, -5e-9), np.array((0, 0, 0)), None, (0.5e-9, 1e-9, 5e-9)],
+            [(-1.5e-9, -5e-9, -5e-9), np.array((0, 0, 0)), (5, 5, 7), None],
+            [[0, 5e-6, 0], (-1.5e-6, -5e-6, -5e-6), None, (0.5e-6, 2e-6, 2.5e-6)],
+            [[0, 5e-6, 0], (-1.5e-6, -5e-6, -5e-6), (1, 10, 20), None],
+            [(0, 125e-9, 0), (500e-9, 0, -3e-9), None, (25e-9, 25e-9, 3e-9)],
+        ]
 
-        self.invalid_args = [[(0, 0, 0), (5, 5, 5),
-                              None, (-1, 1, 1)],
-                             [(0, 0, 0), (5, 5, 5),
-                              (-1, 1, 1), None],
-                             [(0, 0, 0), (5, 5, 5),
-                              'n', None],
-                             [(0, 0, 0), (5, 5, 5),
-                              (1, 2, 2+1j), None],
-                             [(0, 0, 0), (5, 5, 5),
-                              (1, 2, '2'), None],
-                             [('1', 0, 0), (1, 1, 1),
-                              None, (0, 0, 1e-9)],
-                             [(-1.5e-9, -5e-9, 'a'), (1.5e-9, 15e-9, 16e-9),
-                              None, (5, 1, -1e-9)],
-                             [(-1.5e-9, -5e-9, 'a'), (1.5e-9, 15e-9, 16e-9),
-                              (5, 1, -1), None],
-                             [(-1.5e-9, -5e-9, 0), (1.5e-9, 16e-9),
-                              None, (0.1e-9, 0.1e-9, 1e-9)],
-                             [(-1.5e-9, -5e-9, 0), (1.5e-9, 15e-9, 1+2j),
-                              None, (5, 1, 1e-9)],
-                             ['string', (5, 1, 1e-9),
-                              None, 'string'],
-                             [(-1.5e-9, -5e-9, 0), (1.5e-9, 15e-9, 16e-9),
-                              None, 2+2j]]
+        self.invalid_args = [
+            [(0, 0, 0), (5, 5, 5), None, (-1, 1, 1)],
+            [(0, 0, 0), (5, 5, 5), (-1, 1, 1), None],
+            [(0, 0, 0), (5, 5, 5), "n", None],
+            [(0, 0, 0), (5, 5, 5), (1, 2, 2 + 1j), None],
+            [(0, 0, 0), (5, 5, 5), (1, 2, "2"), None],
+            [("1", 0, 0), (1, 1, 1), None, (0, 0, 1e-9)],
+            [(-1.5e-9, -5e-9, "a"), (1.5e-9, 15e-9, 16e-9), None, (5, 1, -1e-9)],
+            [(-1.5e-9, -5e-9, "a"), (1.5e-9, 15e-9, 16e-9), (5, 1, -1), None],
+            [(-1.5e-9, -5e-9, 0), (1.5e-9, 16e-9), None, (0.1e-9, 0.1e-9, 1e-9)],
+            [(-1.5e-9, -5e-9, 0), (1.5e-9, 15e-9, 1 + 2j), None, (5, 1, 1e-9)],
+            ["string", (5, 1, 1e-9), None, "string"],
+            [(-1.5e-9, -5e-9, 0), (1.5e-9, 15e-9, 16e-9), None, 2 + 2j],
+        ]
 
     def test_init_valid_args(self):
         for p1, p2, n, cell in self.valid_args:
@@ -128,17 +133,19 @@ class TestMesh:
     def test_init_invalid_args(self):
         for p1, p2, n, cell in self.invalid_args:
             with pytest.raises((TypeError, ValueError)):
-                mesh = df.Mesh(region=df.Region(p1=p1, p2=p2), n=n, cell=cell)
+                df.Mesh(region=df.Region(p1=p1, p2=p2), n=n, cell=cell)
 
             with pytest.raises((TypeError, ValueError)):
-                mesh = df.Mesh(p1=p1, p2=p2, n=n, cell=cell)
+                df.Mesh(p1=p1, p2=p2, n=n, cell=cell)
 
     def test_init_subregions(self):
         p1 = (0, 0, 0)
         p2 = (100, 50, 10)
         cell = (10, 10, 10)
-        subregions = {'r1': df.Region(p1=(0, 0, 0), p2=(50, 50, 10)),
-                      'r2': df.Region(p1=(50, 0, 0), p2=(100, 50, 10))}
+        subregions = {
+            "r1": df.Region(p1=(0, 0, 0), p2=(50, 50, 10)),
+            "r2": df.Region(p1=(50, 0, 0), p2=(100, 50, 10)),
+        }
         mesh = df.Mesh(p1=p1, p2=p2, cell=cell, subregions=subregions)
         check_mesh(mesh)
 
@@ -148,18 +155,17 @@ class TestMesh:
         cell = (10e-9, 10e-9, 10e-9)
 
         # Subregion not an aggregate.
-        subregions = {'r1': df.Region(p1=(0, 0, 0), p2=(45e-9, 50e-9, 10e-9))}
+        subregions = {"r1": df.Region(p1=(0, 0, 0), p2=(45e-9, 50e-9, 10e-9))}
         with pytest.raises(ValueError):
             mesh = df.Mesh(p1=p1, p2=p2, cell=cell, subregions=subregions)
 
         # Subregion not aligned.
-        subregions = {'r1': df.Region(p1=(5e-9, 0, 0),
-                                      p2=(45e-9, 50e-9, 10e-9))}
+        subregions = {"r1": df.Region(p1=(5e-9, 0, 0), p2=(45e-9, 50e-9, 10e-9))}
         with pytest.raises(ValueError):
             mesh = df.Mesh(p1=p1, p2=p2, cell=cell, subregions=subregions)
 
         # Subregion not in the mesh region.
-        subregions = {'r1': df.Region(p1=(0, 0, 0), p2=(45e-9, 50e-9, 200e-9))}
+        subregions = {"r1": df.Region(p1=(0, 0, 0), p2=(45e-9, 50e-9, 200e-9))}
         with pytest.raises(ValueError):
             mesh = df.Mesh(p1=p1, p2=p2, cell=cell, subregions=subregions)
 
@@ -170,8 +176,8 @@ class TestMesh:
         n = (10, 10, 10)
 
         with pytest.raises(ValueError) as excinfo:
-            mesh = df.Mesh(region=region, p1=p1, p2=p2, n=n)
-        assert 'not both.' in str(excinfo.value)
+            df.Mesh(region=region, p1=p1, p2=p2, n=n)
+        assert "not both." in str(excinfo.value)
 
     def test_init_with_n_and_cell(self):
         p1 = (0, -4, 16.5)
@@ -180,35 +186,33 @@ class TestMesh:
         cell = (1, 0.1, 0.5)
 
         with pytest.raises(ValueError) as excinfo:
-            mesh = df.Mesh(p1=p1, p2=p2, n=n, cell=cell)
-        assert 'not both.' in str(excinfo.value)
+            df.Mesh(p1=p1, p2=p2, n=n, cell=cell)
+        assert "not both." in str(excinfo.value)
 
     def test_region_not_aggregate_of_cell(self):
-        args = [[(0, 100e-9, 1e-9),
-                 (150e-9, 120e-9, 6e-9),
-                 (4e-9, 1e-9, 1e-9)],
-                [(0, 100e-9, 0),
-                 (150e-9, 104e-9, 1e-9),
-                 (2e-9, 1.5e-9, 0.1e-9)],
-                [(10e9, 10e3, 0),
-                 (11e9, 11e3, 5),
-                 (1e9, 1e3, 1.5)]]
+        args = [
+            [(0, 100e-9, 1e-9), (150e-9, 120e-9, 6e-9), (4e-9, 1e-9, 1e-9)],
+            [(0, 100e-9, 0), (150e-9, 104e-9, 1e-9), (2e-9, 1.5e-9, 0.1e-9)],
+            [(10e9, 10e3, 0), (11e9, 11e3, 5), (1e9, 1e3, 1.5)],
+        ]
 
         for p1, p2, cell in args:
-            with pytest.raises(ValueError) as excinfo:
-                mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
+            with pytest.raises(ValueError):
+                df.Mesh(p1=p1, p2=p2, cell=cell)
 
     def test_cell_greater_than_domain(self):
         p1 = (0, 0, 0)
         p2 = (1e-9, 1e-9, 1e-9)
-        args = [(2e-9, 1e-9, 1e-9),
-                (1e-9, 2e-9, 1e-9),
-                (1e-9, 1e-9, 2e-9),
-                (1e-9, 5e-9, 0.1e-9)]
+        args = [
+            (2e-9, 1e-9, 1e-9),
+            (1e-9, 2e-9, 1e-9),
+            (1e-9, 1e-9, 2e-9),
+            (1e-9, 5e-9, 0.1e-9),
+        ]
 
         for cell in args:
-            with pytest.raises(ValueError) as excinfo:
-                mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
+            with pytest.raises(ValueError):
+                df.Mesh(p1=p1, p2=p2, cell=cell)
 
     def test_len(self):
         p1 = (0, 0, 0)
@@ -217,7 +221,7 @@ class TestMesh:
         mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
         check_mesh(mesh)
 
-        assert len(mesh) == 5*4*3
+        assert len(mesh) == 5 * 4 * 3
 
     def test_indices_coordinates_iter(self):
         p1 = (0, 0, 0)
@@ -252,7 +256,7 @@ class TestMesh:
         assert mesh1 == mesh2
         assert not mesh1 != mesh2
         assert mesh1 != 1
-        assert not mesh2 == 'mesh2'
+        assert not mesh2 == "mesh2"
 
         p1 = (0, 0, 0)
         p2 = (10e-9, 5e-9, 3e-9)
@@ -270,13 +274,14 @@ class TestMesh:
         p2 = (15, 10.1, 12.5)
         cell = (1, 0.1, 0.5)
 
-        mesh = df.Mesh(p1=p1, p2=p2, cell=cell, bc='x')
+        mesh = df.Mesh(p1=p1, p2=p2, cell=cell, bc="x")
         check_mesh(mesh)
 
-        rstr = ("Mesh(region=Region(p1=(-1.0, -4.0, 11.0), "
-                "p2=(15.0, 10.1, 12.5)), n=(16, 141, 3), "
-                "bc='x', subregions={}, attributes={'unit': 'm', "
-                "'fourierspace': False, 'isplane': False})")
+        rstr = (
+            "Mesh(Region(p1=(-1, -4, 11), p2=(15, 10.1, 12.5)), "
+            "n=(16, 141, 3), bc=x, attributes: (unit: m, fourierspace: "
+            "False, isplane: False))"
+        )
         assert repr(mesh) == rstr
 
     def test_index2point(self):
@@ -342,17 +347,17 @@ class TestMesh:
 
         tol = 1e-12  # picometer tolerance
         with pytest.raises(ValueError):
-            mesh.point2index((-10-tol, 0, 5))
+            mesh.point2index((-10 - tol, 0, 5))
         with pytest.raises(ValueError):
-            mesh.point2index((-5, -5-tol, 5))
+            mesh.point2index((-5, -5 - tol, 5))
         with pytest.raises(ValueError):
             mesh.point2index((-5, 0, -tol))
         with pytest.raises(ValueError):
-            mesh.point2index((10+tol, 0, 5))
+            mesh.point2index((10 + tol, 0, 5))
         with pytest.raises(ValueError):
-            mesh.point2index((6, 5+tol, 5))
+            mesh.point2index((6, 5 + tol, 5))
         with pytest.raises(ValueError):
-            mesh.point2index((0, 0, 10e-9+tol))
+            mesh.point2index((0, 0, 10e-9 + tol))
 
     def test_index2point_point2index_mutually_inverse(self):
         p1 = (15, -4, 12.5)
@@ -367,15 +372,60 @@ class TestMesh:
         for i in [(1, 0, 0), (0, 1, 0), (0, 0, 1), (1, 1, 1)]:
             assert mesh.point2index(mesh.index2point(i)) == i
 
+    def test_region2slice(self):
+        p1 = (0, 0, -2)
+        p2 = (4, 5, 4)
+        cell = (1, 1, 1)
+        mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
+        check_mesh(mesh)
+        assert mesh.region2slices(df.Region(p1=p1, p2=p2)) == (
+            slice(0, 4, None),
+            slice(0, 5, None),
+            slice(0, 6, None),
+        )
+        assert mesh.region2slices(df.Region(p1=(0, 0, 0), p2=(1, 1, 1))) == (
+            slice(0, 1, None),
+            slice(0, 1, None),
+            slice(2, 3, None),
+        )
+        assert mesh.region2slices(df.Region(p1=(2, 3, -1), p2=(3, 5, 0))) == (
+            slice(2, 3, None),
+            slice(3, 5, None),
+            slice(1, 2, None),
+        )
+        with pytest.raises(ValueError):
+            mesh.region2slices(df.Region(p1=(-1, 3, -1), p2=(3, 5, 0)))
+
+    @pytest.mark.filterwarnings("ignore::FutureWarning")
     def test_axis_points(self):
         p1 = (0, 0, 0)
         p2 = (10, 6, 8)
         cell = (2, 2, 2)
         mesh = df.Mesh(region=df.Region(p1=p1, p2=p2), cell=cell)
 
-        assert list(mesh.axis_points('x')) == [1.0, 3.0, 5.0, 7.0, 9.0]
-        assert list(mesh.axis_points('y')) == [1.0, 3.0, 5.0]
-        assert list(mesh.axis_points('z')) == [1.0, 3.0, 5.0, 7.0]
+        assert np.allclose(mesh.axis_points("x"), [1.0, 3.0, 5.0, 7.0, 9.0])
+        assert np.allclose(mesh.axis_points("y"), [1.0, 3.0, 5.0])
+        assert np.allclose(mesh.axis_points("z"), [1.0, 3.0, 5.0, 7.0])
+
+    def test_midpoints(self):
+        p1 = (0, 0, 4)
+        p2 = (10, 6, 0)
+        cell = (2, 2, 1)
+        mesh = df.Mesh(region=df.Region(p1=p1, p2=p2), cell=cell)
+
+        assert np.allclose(mesh.midpoints.x, [1.0, 3.0, 5.0, 7.0, 9.0])
+        assert np.allclose(mesh.midpoints.y, [1.0, 3.0, 5.0])
+        assert np.allclose(mesh.midpoints.z, [0.5, 1.5, 2.5, 3.5])
+
+    def test_vertices(self):
+        p1 = (0, 1, 0)
+        p2 = (5, 0, 6)
+        cell = (1, 1, 2)
+        mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
+
+        assert np.allclose(mesh.vertices.x, [0.0, 1.0, 2.0, 3.0, 4.0, 5.0])
+        assert np.allclose(mesh.vertices.y, [0.0, 1.0])
+        assert np.allclose(mesh.vertices.z, [0.0, 2.0, 4.0, 6.0])
 
     def test_neighbours(self):
         p1 = (0, 0, 0)
@@ -402,7 +452,7 @@ class TestMesh:
         p1 = (0, 0, 0)
         p2 = (5, 5, 5)
         n = (5, 5, 5)
-        mesh = df.Mesh(region=df.Region(p1=p1, p2=p2), n=n, bc='xy')
+        mesh = df.Mesh(region=df.Region(p1=p1, p2=p2), n=n, bc="xy")
 
         neighbours = mesh.neighbours((0, 0, 0))
         assert isinstance(neighbours, list)
@@ -432,7 +482,6 @@ class TestMesh:
         mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
         check_mesh(mesh)
 
-        tol = 1e-12
         line = mesh.line(p1=(0, 0, 0), p2=(10, 10, 10), n=10)
         assert isinstance(line, types.GeneratorType)
         assert len(list(line)) == 10
@@ -478,7 +527,7 @@ class TestMesh:
             assert len(point) == 3
             assert point[1] == 4.2
 
-        plane = mesh.plane('x')
+        plane = mesh.plane("x")
         check_mesh(plane)
         assert isinstance(plane, df.Mesh)
         assert len(list(plane)) == 15
@@ -487,7 +536,7 @@ class TestMesh:
             assert len(point) == 3
             assert point[0] == 5
 
-        plane = mesh.plane('y', n=(10, 10))
+        plane = mesh.plane("y", n=(10, 10))
         check_mesh(plane)
         assert isinstance(plane, df.Mesh)
         assert len(list(plane)) == 100
@@ -506,77 +555,77 @@ class TestMesh:
             plane = list(mesh.plane(z=-1e-9))
 
         with pytest.raises(ValueError):
-            plane = list(mesh.plane('x', z=1))
+            plane = list(mesh.plane("x", z=1))
 
         with pytest.raises(ValueError):
-            plane = list(mesh.plane('z', z=1))
+            plane = list(mesh.plane("z", z=1))
 
         with pytest.raises(ValueError):
             plane = list(mesh.plane(x=2, z=1))
 
-        info = mesh.plane('x').attributes
-        assert info['isplane']
-        assert info['planeaxis'] == 0
-        assert info['axis1'] == 1
-        assert info['axis2'] == 2
-        assert info['point'] == 5
+        info = mesh.plane("x").attributes
+        assert info["isplane"]
+        assert info["planeaxis"] == 0
+        assert info["axis1"] == 1
+        assert info["axis2"] == 2
+        assert info["point"] == 5
 
-        info = mesh.plane('y').attributes
-        assert info['isplane']
-        assert info['planeaxis'] == 1
-        assert info['axis1'] == 0
-        assert info['axis2'] == 2
-        assert info['point'] == 2.5
+        info = mesh.plane("y").attributes
+        assert info["isplane"]
+        assert info["planeaxis"] == 1
+        assert info["axis1"] == 0
+        assert info["axis2"] == 2
+        assert info["point"] == 2.5
 
-        info = mesh.plane('z').attributes
-        assert info['isplane']
-        assert info['planeaxis'] == 2
-        assert info['axis1'] == 0
-        assert info['axis2'] == 1
-        assert info['point'] == 1.5
+        info = mesh.plane("z").attributes
+        assert info["isplane"]
+        assert info["planeaxis"] == 2
+        assert info["axis1"] == 0
+        assert info["axis2"] == 1
+        assert info["point"] == 1.5
 
         info = mesh.plane(x=0).attributes
-        assert info['isplane']
-        assert info['planeaxis'] == 0
-        assert info['axis1'] == 1
-        assert info['axis2'] == 2
-        assert info['point'] == 0
+        assert info["isplane"]
+        assert info["planeaxis"] == 0
+        assert info["axis1"] == 1
+        assert info["axis2"] == 2
+        assert info["point"] == 0
 
         info = mesh.plane(y=0).attributes
-        assert info['isplane']
-        assert info['planeaxis'] == 1
-        assert info['axis1'] == 0
-        assert info['axis2'] == 2
-        assert info['point'] == 0
+        assert info["isplane"]
+        assert info["planeaxis"] == 1
+        assert info["axis1"] == 0
+        assert info["axis2"] == 2
+        assert info["point"] == 0
 
         info = mesh.plane(z=0).attributes
-        assert info['isplane']
-        assert info['planeaxis'] == 2
-        assert info['axis1'] == 0
-        assert info['axis2'] == 1
-        assert info['point'] == 0
+        assert info["isplane"]
+        assert info["planeaxis"] == 2
+        assert info["axis1"] == 0
+        assert info["axis2"] == 1
+        assert info["point"] == 0
 
         info = mesh.plane(x=5).attributes
-        assert info['isplane']
-        assert info['planeaxis'] == 0
-        assert info['axis1'] == 1
-        assert info['axis2'] == 2
-        assert info['point'] == 5
+        assert info["isplane"]
+        assert info["planeaxis"] == 0
+        assert info["axis1"] == 1
+        assert info["axis2"] == 2
+        assert info["point"] == 5
 
         with pytest.raises(KeyError):
-            plane_mesh = mesh.plane('xy')
+            mesh.plane("xy")
         with pytest.raises(KeyError):
-            plane_mesh = mesh.plane('zy')
+            mesh.plane("zy")
         with pytest.raises(ValueError):
-            plane_mesh = mesh.plane('y', 'x')
+            mesh.plane("y", "x")
         with pytest.raises(KeyError):
-            plane_mesh = mesh.plane('xzy')
+            mesh.plane("xzy")
         with pytest.raises(ValueError):
-            plane_mesh = mesh.plane('z', x=3)
+            mesh.plane("z", x=3)
         with pytest.raises(ValueError):
-            plane_mesh = mesh.plane('y', y=5)
+            mesh.plane("y", y=5)
         with pytest.raises(ValueError):
-            plane_mesh = mesh.plane('z', x=5)
+            mesh.plane("z", x=5)
 
     def test_or(self):
         p1 = (-50e-9, -25e-9, 0)
@@ -609,18 +658,20 @@ class TestMesh:
         p1 = (0, 0, 0)
         p2 = (100, 50, 10)
         cell = (5, 5, 5)
-        subregions = {'r1': df.Region(p1=(0, 0, 0), p2=(50, 50, 10)),
-                      'r2': df.Region(p1=(50, 0, 0), p2=(100, 50, 10))}
+        subregions = {
+            "r1": df.Region(p1=(0, 0, 0), p2=(50, 50, 10)),
+            "r2": df.Region(p1=(50, 0, 0), p2=(100, 50, 10)),
+        }
         mesh = df.Mesh(p1=p1, p2=p2, cell=cell, subregions=subregions)
         check_mesh(mesh)
 
-        submesh1 = mesh['r1']
+        submesh1 = mesh["r1"]
         check_mesh(submesh1)
         assert submesh1.region.pmin == (0, 0, 0)
         assert submesh1.region.pmax == (50, 50, 10)
         assert submesh1.cell == (5, 5, 5)
 
-        submesh2 = mesh['r2']
+        submesh2 = mesh["r2"]
         check_mesh(submesh2)
         assert submesh2.region.pmin == (50, 0, 0)
         assert submesh2.region.pmax == (100, 50, 10)
@@ -649,20 +700,22 @@ class TestMesh:
         mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
         check_mesh(mesh)
 
-        submesh = mesh[df.Region(p1=(11e-9, 22e-9, 1e-9),
-                                 p2=(-9e-9, 79e-9, 14e-9))]
+        submesh = mesh[df.Region(p1=(11e-9, 22e-9, 1e-9), p2=(-9e-9, 79e-9, 14e-9))]
         check_mesh(submesh)
-        assert np.allclose(submesh.region.pmin, (-10e-9, 20e-9, 0),
-                           atol=1e-15, rtol=1e-5)
-        assert np.allclose(submesh.region.pmax, (15e-9, 80e-9, 15e-9),
-                           atol=1e-15, rtol=1e-5)
+        assert np.allclose(
+            submesh.region.pmin, (-10e-9, 20e-9, 0), atol=1e-15, rtol=1e-5
+        )
+        assert np.allclose(
+            submesh.region.pmax, (15e-9, 80e-9, 15e-9), atol=1e-15, rtol=1e-5
+        )
         assert submesh.cell == cell
         assert submesh.n == (5, 12, 3)
         assert mesh[mesh.region] == mesh
 
         with pytest.raises(ValueError):
-            submesh = mesh[df.Region(p1=(11e-9, 22e-9, 1e-9),
-                                     p2=(200e-9, 79e-9, 14e-9))]
+            submesh = mesh[
+                df.Region(p1=(11e-9, 22e-9, 1e-9), p2=(200e-9, 79e-9, 14e-9))
+            ]
 
     def test_pad(self):
         p1 = (-1, 2, 7)
@@ -671,22 +724,22 @@ class TestMesh:
         region = df.Region(p1=p1, p2=p2)
         mesh = df.Mesh(region=region, cell=cell)
 
-        padded_mesh = mesh.pad({'x': (0, 1)})
+        padded_mesh = mesh.pad({"x": (0, 1)})
         assert padded_mesh.region.pmin == (-1, 2, 4)
         assert padded_mesh.region.pmax == (6, 9, 7)
         assert padded_mesh.n == (7, 7, 3)
 
-        padded_mesh = mesh.pad({'y': (1, 1)})
+        padded_mesh = mesh.pad({"y": (1, 1)})
         assert padded_mesh.region.pmin == (-1, 1, 4)
         assert padded_mesh.region.pmax == (5, 10, 7)
         assert padded_mesh.n == (6, 9, 3)
 
-        padded_mesh = mesh.pad({'z': (2, 3)})
+        padded_mesh = mesh.pad({"z": (2, 3)})
         assert padded_mesh.region.pmin == (-1, 2, 2)
         assert padded_mesh.region.pmax == (5, 9, 10)
         assert padded_mesh.n == (6, 7, 8)
 
-        padded_mesh = mesh.pad({'x': (1, 1), 'y': (1, 1), 'z': (1, 1)})
+        padded_mesh = mesh.pad({"x": (1, 1), "y": (1, 1), "z": (1, 1)})
         assert padded_mesh.region.pmin == (-2, 1, 3)
         assert padded_mesh.region.pmax == (6, 10, 8)
         assert padded_mesh.n == (8, 9, 5)
@@ -702,7 +755,7 @@ class TestMesh:
         assert mesh.dz == 10e-9
 
         with pytest.raises(AttributeError):
-            res = mesh.dk
+            mesh.dk
 
     def test_dir(self):
         p1 = (0, 0, 0)
@@ -710,7 +763,7 @@ class TestMesh:
         cell = (1e-9, 5e-9, 10e-9)
         mesh = df.Mesh(region=df.Region(p1=p1, p2=p2), cell=cell)
 
-        assert all([i in dir(mesh) for i in ['dx', 'dy', 'dz']])
+        assert all([i in dir(mesh) for i in ["dx", "dy", "dz"]])
 
     def test_dV(self):
         p1 = (0, 0, 0)
@@ -726,55 +779,58 @@ class TestMesh:
         cell = (1, 2, 2.5)
         mesh = df.Mesh(region=df.Region(p1=p1, p2=p2), cell=cell)
 
-        assert mesh.plane('x').dS.average == (5, 0, 0)
-        assert mesh.plane('y').dS.average == (0, 2.5, 0)
-        assert mesh.plane('z').dS.average == (0, 0, 2)
+        assert mesh.plane("x").dS.average == (5, 0, 0)
+        assert mesh.plane("y").dS.average == (0, 2.5, 0)
+        assert mesh.plane("z").dS.average == (0, 0, 2)
 
         # Exception
         with pytest.raises(ValueError):
-            res = mesh.dS
+            mesh.dS
 
     def test_mpl(self):
         for p1, p2, n, cell in self.valid_args:
             mesh = df.Mesh(region=df.Region(p1=p1, p2=p2), n=n, cell=cell)
             mesh.mpl()
+            mesh.mpl(box_aspect=[1, 2, 3])
 
-            filename = 'figure.pdf'
+            filename = "figure.pdf"
             with tempfile.TemporaryDirectory() as tmpdir:
                 tmpfilename = os.path.join(tmpdir, filename)
                 mesh.mpl(filename=tmpfilename)
 
-            plt.close('all')
+            plt.close("all")
 
     def test_k3d(self):
         for p1, p2, n, cell in self.valid_args:
             mesh = df.Mesh(p1=p1, p2=p2, n=n, cell=cell)
             mesh.k3d()
-            mesh.plane('x').k3d()
+            mesh.plane("x").k3d()
 
     def test_k3d_mpl_subregions(self):
         p1 = (0, 0, 0)
         p2 = (100, 80, 10)
         cell = (100, 5, 10)
-        subregions = {'r1': df.Region(p1=(0, 0, 0), p2=(100, 10, 10)),
-                      'r2': df.Region(p1=(0, 10, 0), p2=(100, 20, 10)),
-                      'r3': df.Region(p1=(0, 20, 0), p2=(100, 30, 10)),
-                      'r4': df.Region(p1=(0, 30, 0), p2=(100, 40, 10)),
-                      'r5': df.Region(p1=(0, 40, 0), p2=(100, 50, 10)),
-                      'r6': df.Region(p1=(0, 50, 0), p2=(100, 60, 10)),
-                      'r7': df.Region(p1=(0, 60, 0), p2=(100, 70, 10)),
-                      'r8': df.Region(p1=(0, 70, 0), p2=(100, 80, 10))}
+        subregions = {
+            "r1": df.Region(p1=(0, 0, 0), p2=(100, 10, 10)),
+            "r2": df.Region(p1=(0, 10, 0), p2=(100, 20, 10)),
+            "r3": df.Region(p1=(0, 20, 0), p2=(100, 30, 10)),
+            "r4": df.Region(p1=(0, 30, 0), p2=(100, 40, 10)),
+            "r5": df.Region(p1=(0, 40, 0), p2=(100, 50, 10)),
+            "r6": df.Region(p1=(0, 50, 0), p2=(100, 60, 10)),
+            "r7": df.Region(p1=(0, 60, 0), p2=(100, 70, 10)),
+            "r8": df.Region(p1=(0, 70, 0), p2=(100, 80, 10)),
+        }
         mesh = df.Mesh(p1=p1, p2=p2, cell=cell, subregions=subregions)
 
         # matplotlib tests
-        mesh.mpl_subregions()
+        mesh.mpl_subregions(box_aspect=(1, 1, 1), show_region=True)
 
-        filename = 'figure.pdf'
+        filename = "figure.pdf"
         with tempfile.TemporaryDirectory() as tmpdir:
             tmpfilename = os.path.join(tmpdir, filename)
             mesh.mpl_subregions(filename=tmpfilename)
 
-        plt.close('all')
+        plt.close("all")
 
         # k3d tests
         mesh.k3d_subregions()
@@ -786,14 +842,14 @@ class TestMesh:
         mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
         check_mesh(mesh)
 
-        x_slider = mesh.slider('x')
+        x_slider = mesh.slider("x")
         assert isinstance(x_slider, ipywidgets.SelectionSlider)
 
-        y_slider = mesh.slider('y', multiplier=1)
-        assert isinstance(x_slider, ipywidgets.SelectionSlider)
+        y_slider = mesh.slider("y", multiplier=1)
+        assert isinstance(y_slider, ipywidgets.SelectionSlider)
 
-        z_slider = mesh.slider('z', multiplier=1e3)
-        assert isinstance(x_slider, ipywidgets.SelectionSlider)
+        z_slider = mesh.slider("z", multiplier=1e3)
+        assert isinstance(z_slider, ipywidgets.SelectionSlider)
 
     def test_axis_selector(self):
         p1 = (-10e-9, -5e-9, 10e-9)
@@ -805,11 +861,11 @@ class TestMesh:
         axis_widget = mesh.axis_selector()
         assert isinstance(axis_widget, ipywidgets.Dropdown)
 
-        axis_widget = mesh.axis_selector(widget='radiobuttons')
+        axis_widget = mesh.axis_selector(widget="radiobuttons")
         assert isinstance(axis_widget, ipywidgets.RadioButtons)
 
-        axis_widget = mesh.axis_selector(description='something')
+        axis_widget = mesh.axis_selector(description="something")
         assert isinstance(axis_widget, ipywidgets.Dropdown)
 
         with pytest.raises(ValueError):
-            axis_widget = mesh.axis_selector(widget='something')
+            axis_widget = mesh.axis_selector(widget="something")
