@@ -1,6 +1,5 @@
-import pytest
 import numpy as np
-import discretisedfield as df
+
 import discretisedfield.util as dfu
 
 
@@ -14,22 +13,20 @@ def test_bergluescher_angle():
     v2 = (0, 1, 0)
     v3 = (0, 0, 1)
 
-    angle = dfu.bergluescher_angle(v1, v2, v3)
     # CCW orientation
-    assert dfu.bergluescher_angle(v1, v2, v3) == 1/8
-    assert dfu.bergluescher_angle(v2, v3, v1) == 1/8
-    assert dfu.bergluescher_angle(v3, v1, v2) == 1/8
+    assert dfu.bergluescher_angle(v1, v2, v3) == 1 / 8
+    assert dfu.bergluescher_angle(v2, v3, v1) == 1 / 8
+    assert dfu.bergluescher_angle(v3, v1, v2) == 1 / 8
     # CW orientation
-    assert dfu.bergluescher_angle(v3, v2, v1) == -1/8
-    assert dfu.bergluescher_angle(v2, v1, v3) == -1/8
-    assert dfu.bergluescher_angle(v1, v3, v2) == -1/8
+    assert dfu.bergluescher_angle(v3, v2, v1) == -1 / 8
+    assert dfu.bergluescher_angle(v2, v1, v3) == -1 / 8
+    assert dfu.bergluescher_angle(v1, v3, v2) == -1 / 8
 
     # 0 of the full angle
     v1 = (1, 0, 0)
     v2 = (1, 0, 0)
     v3 = (0, 0, 1)
 
-    angle = dfu.bergluescher_angle(v1, v2, v3)
     assert dfu.bergluescher_angle(v1, v2, v3) == 0
     assert dfu.bergluescher_angle(v2, v3, v1) == 0
     assert dfu.bergluescher_angle(v3, v1, v2) == 0
